%> @file ParamCurve.m
%> @brief Contains the geom.ParamCurve class.
% ======================================================================
%> @brief Smooth curve parametrized over an interval.
%>
%> This class is abstract as it does not provide any concrete
%> implementation of the parametrization. It only specifies an abstract
%> interface for evaluation of the parametrization. 
%>
%> The evaluation takes account of any geometrical transformations applied
%> to the curve. These each specify an affine transform
%> @f[
%>    \alpha(x) = A \, x + b \, .
%> @f]
%> The ParamCurve class accumulates the effects of multiple such
%> transformations and applies them when evaluating the parametrization.
%>
%> The plot functionality is also implemented.
%>
classdef ParamCurve < BIEPack.geom.Curve
    
    properties (SetAccess = protected)
        
        %> @brief Smoothness of the parametrization of the curve.
        %> At least 1, can be infinity.
        smoothness = 1;
        
        %> @brief The degree of the highest derivative that can be computed
        %> using the eval()-method.
        highestImplementedDerivative = 1;
        
        %> @brief Lower boundary of the parameter interval.
        startParam = 0;
        
        %> @brief Upper boundary of the parameter interval.
        endParam = 1;
        
        %> @brief Matrix of the affine transformation generated by geometrical
        %> transformation. Is is used when evaluating the parametrization.
        matrixA = eye(2);
        
        %> @brief Vector of the affine transformationgenerated by geometrical
        %> transformation. Is is used when evaluating the parametrization.
        vectorB = zeros(2,1);
        
    end
    
    methods (Abstract)
        
        %> @brief Evaluates the n-th derivative of the parametrization 
        %> for parameter values t before the application of the affine 
        %> transformation.
        %>
        %> @param t Parameter values for which the parametrization
        %> will be evaluated. May be a matrix.
        %> @param n Order of the derivative that will be evaluated.
        %> 
        %> @retval y1 Value of the 1st coordiante of the n-th derivative at
        %> t. Has the same dimension as t.
        %> @retval y2 Value of the 2nd coordiante of the n-th derivative at
        %> t. Has the same dimension as t.
        [y1,y2] = paramEval(curve,t,n);
        
    end
    
    methods
        
        %> @brief Evaluates the n-th derivative of the parametrization for
        %> parameter values t, after applying the affine transformation
        %> represented by matrixA and vectorB.
        
        %> @param curve Curve object for which the the values will be
        %> evaluated.
        %> @param t Parameter values for which the parametrization
        %> will be evaluated. May be a matrix.
        %> @param n Order of the derivative that will be evaluated.
        %> 
        %> @retval y1 Value of the x1-coordiante of the n-th derivative at
        %> t. Has the same dimension as t.
        %> @retval y1 Value of the x2-coordiante of the n-th derivative at
        %> t. Has the same dimension as t.
        function [y1, y2] = eval(curve, t, n)
            
            T = t(:);
            
            if ( n < 0 || ( round(n) ~= n ) )
                error( 'In ParamCurve %s, method eval: parameter n is not a non-negative integer.', curve.name );
            end
            
            if ( n > curve.highestImplementedDerivative )
                error( 'In ParamCurve %s, method eval: evaluation of %d-th derivative not implemented.', curve.name, n );
            end
            
            if ( any(T < curve.startParam) || any(T > curve.endParam) )
                error( 'In ParamCurve %s, method eval: t outside param interval.', curve.name );
            end
            
            [y1_tmp, y2_tmp] = curve.paramEval(t,n);
            
            y1 = curve.matrixA(1,1) * y1_tmp + curve.matrixA(1,2) * y2_tmp;
            y2 = curve.matrixA(2,1) * y1_tmp + curve.matrixA(2,2) * y2_tmp;
            
            if ( n == 0 )
                y1 = y1 + curve.vectorB(1);
                y2 = y2 + curve.vectorB(2);
            end
            
        end
        
        %> @brief Obtains the non-normalized normal vector to a curve at
        %> the points t.
        %>
        %> @param curve Curve object whose normal vectors will be obtained.
        %> @param t Parameter values at wich the normal vector will be evaluated. May
        %> be matrix.
        %>
        %> @retval n1 1st coordinate of the normal vector at t. Has the same dimension as t.
        %> @retval n2 2nd coordinate of the normal vector at t. Has the same dimension as t.
        function [n1, n2] = normal(curve, t)
            
            [t1, t2] = curve.eval(t,1);
            n1 = t2;
            n2 = -t1;
            
        end
        
        %> @brief The ParamCurve object consists of just one segment:
        %> itself.
        function listOfCurves = getParamCurveList(curve)
            
            listOfCurves = curve;
            
        end
                
        %> @brief Implements translation by adding translation vector to
        %> vectorB
        function h = translate(curve,d)
            
            curve.vectorB = curve.vectorB + d;            
            
            h = curve;
            
        end
        
        %> @brief Implements rotation by multiplying matrixA with a rotation
        %> matrix.
        function h = rotate(curve, phi, p)
            
            if (nargin == 2)
                p = [0;0];
            end
           
            % with 
            %  y = b + Ax
            % and the rotation around p given by
            %  z = p + J(y - p)
            % we obtain
            %  z = p + J(b - p) + JA x
            J = [ cos(phi), -sin(phi); sin(phi), cos(phi) ];
            
            curve.vectorB = p + J*(curve.vectorB - p);
            curve.matrixA = J * curve.matrixA;
            
            h = curve;
            
        end
        
        %> @brief Implements reflection by multiplying matrixA with a rotation
        %> matrix.
        function h = reflect(curve, varargin)
            
            if ( nargin == 1 )
                
                % reflection at origin
                curve.vectorB = - curve.vectorB;
                curve.matrixA = - curve.matrixA;
                
            elseif ( nargin == 2 )
                
                % reflection at point
                curve.vectorB = 2*varargin{1} - curve.vectorB;
                curve.matrixA = -curve.matrixA;
                
            else
                
                % reflection at line varargin{1} + t varargin{2}
                a = [ varargin{2}(2); -varargin{2}(1) ] / norm( varargin{2}, 2 );
                J = eye(2) - 2 * a * a.';
                curve.vectorB = varargin{1} + J*(curve.vectorB - varargin{1});
                curve.matrixA = J * curve.matrixA;
                
            end
            
            h = curve;
            
        end
        
        %> @brief Plots the curve with N points in a line plot.
        function plot(curve, N)
            
            t = linspace(curve.startParam, curve.endParam, N);
            [y1, y2] = curve.eval(t,0);
            plot(y1,y2);
            
        end
        
    end
    
end

