%> @file Mesh.m
%> @brief Contains the meshes.Mesh class.
% ======================================================================
%> @brief Abstract super class for all meshes
%>
%> Each mesh consists of nodes (points in the plane) and elements 
%> (d-dimensional objects connecting certain nodes, this might be 
%> boundary elements or domain elements). A finer mesh may be
%> generated from a coarser mesh by some refinement procedure.
%>
%> The abstract Mesh class provides the interface for this
%> functionality. It stores a list of (x,y)-coordinates for the nodes
%> and information on which node belongs to which element. From this,
%> queries regarding neighbors or connectivity canbe answered.
%>
%> Further geometrical information such as which node/element is
%> situated on which geometrical entity is left to subsclasses.
%>
%> Various methods of refinement exist depending on the concrete mesh
%> implementation. Such refinements are described in objects of the
%> class RefinementRelation. A mesh generated by refinement will store
%> a handle to the corresponding RefinementRelation which in turn has a
%> handle to the coarser mesh.
%>
%> Mesh consisting of submeshes?
% ======================================================================
classdef Mesh < BIEPack.BIEPackObject
    % BIEPack.meshes.Mesh < BIEPack.BIEPackObject
    
    properties (SetAccess = protected)
        
        %> @brief Total number of nodes in the mesh
        numNodes = 0;
        
        %> @brief The x1-coordinates of the nodes
        nodesX = [];
        
        %> @brief The x2-coordinates of the nodes
        nodesY = [];
        
        %> @brief Total number of elements in the mesh
        numElements = 0;
        

        %> @brief Matrix that contains the definition of the elements. The
        %> j-th element of the mesh is represented by the j-th column. 
        %> The first row contains the number of nodes for the j-th element, 
        %> followed by the node indeces in consecutive rows. 
        %> Remaining entries are zero.
        elements = [];
        
        %> @brief Whenever the mesh has been generated from refining another 
        %> mesh, this object describes the refinement.
        refinementRelation = [];
    end
    
    methods
        
        %> @brief This function defines the coordinates of the nodes. 
        %> Both arguments should be row vectors of equal size.
        %>
        %> @param xcoor x1-coordinates of the nodes.
        %> @param ycoor x2-coordinates of the nodes.
        function setNodes( mesh, xcoor, ycoor )
                        
            if ( size(xcoor,2) ~= size(ycoor,2) )
                error( 'In Mesh %s, method setNodes: not the same number of x and y coordinates.', mesh.name );
            end
                        
            if ( size(xcoor,1) ~= 1 || size(ycoor,1) ~= 1 )
                error( 'In Mesh %s, method setNodes: x and y coordinates must be row vectors.', mesh.name );
            end
            
            mesh.nodesX = xcoor;
            mesh.nodesY = ycoor;
            
            mesh.numNodes = size(xcoor,2);
            
        end
        

        %> @brief Sets the element information, only after the nodes have 
        %> already been defined.
        %>
        %> @param elements Elements matrix as defined in the definition of
        %> the property 'elements'.
        function setElements( mesh, elements )
            
            [m,n] = size(elements);
            
            if ( max( elements(2:m,:) ) > mesh.numNodes )
                error( 'In Mesh %s, method setElements: node index in input exceeds number of defined nodes = %d.', mesh.name, mesh.numNodes );
            end
            
            mesh.elements = elements;
            mesh.numElements = n;
            
        end        
        
    end
    
    methods (Abstract)
        
        % methods for mesh refinement
    end
    
    
end
